/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.routes;

import org.apache.camel.Exchange;
import org.apache.camel.Processor;

/**
 * Common class, implementing {@link Processor}, used by the {@link org.apache.camel.builder.RouteBuilder}s
 * in this package.
 * <p/>
 * User: pabe
 * Date: 2011-08-30
 * Time: 11:19
 */
public class ErrorProcessor implements Processor {

    @Override
    public void process(Exchange exchange) throws Exception {
        Object exception = exchange.getProperty(Exchange.EXCEPTION_CAUGHT);
        if (exception instanceof Throwable) {
            Throwable ex = (Throwable) exception;
            while (ex.getCause() != null) {
                ex = ex.getCause();
            }
            if (ex.getClass().getPackage().getName().startsWith("java.net")) {
                exchange.getOut().setBody(ex);
            } else {
                exchange.getOut().setBody(new Exception(((Throwable) exception).getMessage(), ex));
            }
        } else {
            exchange.getOut().setBody(new Exception("Unknown error"));
        }
    }
}
